@@ -3,7 +3,6 @@
 
 
 """Server for PB benchmark."""
-from __future__ import print_function
 
 from zope.interface import implementer
 
@@ -11,22 +10,24 @@ from twisted.spread import pb
 from twisted.internet import reactor
 from twisted.cred.portal import IRealm
 
+
 class PBBenchPerspective(pb.Avatar):
     callsPerSec = 0
+
     def __init__(self):
         pass
-    
+
     def perspective_simple(self):
         self.callsPerSec = self.callsPerSec + 1
         return None
 
     def printCallsPerSec(self):
-        print('(s) cps:', self.callsPerSec)
+        print("(s) cps:", self.callsPerSec)
         self.callsPerSec = 0
         reactor.callLater(1, self.printCallsPerSec)
 
     def perspective_complexTypes(self):
-        return ['a', 1, 1, 1.0, [], ()]
+        return ["a", 1, 1, 1.0, [], ()]
 
 
 @implementer(IRealm)
@@ -35,7 +36,7 @@ class SimpleRealm:
         if pb.IPerspective in interfaces:
             p = PBBenchPerspective()
             p.printCallsPerSec()
-            return pb.IPerspective, p, lambda : None
+            return pb.IPerspective, p, lambda: None
         else:
             raise NotImplementedError("no interface")
 
@@ -43,12 +44,14 @@ class SimpleRealm:
 def main():
     from twisted.cred.portal import Portal
     from twisted.cred.checkers import InMemoryUsernamePasswordDatabaseDontUse
+
     portal = Portal(SimpleRealm())
     checker = InMemoryUsernamePasswordDatabaseDontUse()
-    checker.addUser("benchmark", "benchmark")
+    checker.addUser(b"benchmark", b"benchmark")
     portal.registerChecker(checker)
     reactor.listenTCP(8787, pb.PBServerFactory(portal))
     reactor.run()
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     main()
