@@ -1,14 +1,17 @@
 from twisted.internet import protocol, reactor, endpoints
 from twisted.protocols import basic
 
+
 class FingerProtocol(basic.LineReceiver):
     def lineReceived(self, user):
-        self.transport.write("No such user\r\n")
+        self.transport.write(b"No such user\r\n")
         self.transport.loseConnection()
 
+
 class FingerFactory(protocol.ServerFactory):
     protocol = FingerProtocol
 
+
 fingerEndpoint = endpoints.serverFromString(reactor, "tcp:1079")
 fingerEndpoint.listen(FingerFactory())
 reactor.run()
