@@ -3,24 +3,29 @@
 from twisted.internet import protocol, reactor, defer, utils, endpoints
 from twisted.protocols import basic
 
+
 class FingerProtocol(basic.LineReceiver):
     def lineReceived(self, user):
         d = self.factory.getUser(user)
 
         def onError(err):
-            return 'Internal error in server'
+            return b"Internal error in server"
+
         d.addErrback(onError)
 
         def writeResponse(message):
-            self.transport.write(message + '\r\n')
+            self.transport.write(message + b"\r\n")
             self.transport.loseConnection()
+
         d.addCallback(writeResponse)
 
+
 class FingerFactory(protocol.ServerFactory):
     protocol = FingerProtocol
 
     def getUser(self, user):
-        return utils.getProcessOutput("finger", [user])
+        return utils.getProcessOutput(b"finger", [user])
+
 
 fingerEndpoint = endpoints.serverFromString(reactor, "tcp:1079")
 fingerEndpoint.listen(FingerFactory())
